
# borg-assmiliator/Binding.pm: BindingDB assimilation perl module
# Copyright (C) 2011 Bradley Worley
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License,
# or (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the
# 
#     Free Software Foundation, Inc.
#     59 Temple Place, Suite 330
#     Boston, MA 02111-1307 USA

# make life difficult.
use warnings;
use strict;

# load our perl modules.
use Cwd;

# declare some filenames.
my $binding_file = "BindingDB2D.sdf";

# declare the hashes.
my %binding_hash;

# binding_parse_db: parses the downloaded bindingdb file.
sub binding_build_hash {
  # get the vars.
  my $cwd = getcwd ();
  my $parmName;
  my $parmVal;

  # vars for saving to the database.
  my $monid;
  my $het;
  my $name;
  my $uniprots = "";

  # output a quick info message.
  print "parsing pre-downloaded binding database.\n";

  # open the binding database.
  open (BINDING, "<", $cwd . "/" . $binding_file);

  # loop through the file.
  while (my $line = <BINDING>) {
    # try to parse out a parameter name.
    if ($line =~ /^> <(.*)>$/) {
      # get the parameter name.
      my $pname = trim ($1);
      $parmName = $pname;
    }
    elsif ($line =~ /^$/) {
      # this is a useless line.
    }
    elsif ($line =~ /^\$\$\$\$$/) {
      # this is the end of a monomer.
      unless ($uniprots eq "") {
        # split the uniprot list string.
        my @uids = split (/ /, $uniprots);

        # loop through the uniprot ids.
        foreach my $uid (@uids) {
          # screw the blank ones.
          next if ($uid eq "");

          # make sure the uniprot exists. does this speed things up?
          next unless uniprot_is_valid ($uid);

          # make sure the uniprot is reviewed and original.
          uniprot_yank_from_db ($uid);
          next unless uniprot_is_reviewed ($uid);

          # if this is an original uniprot, push it to the stack.
          if (uniprot_is_original ($uid)) {
            my $bindage = $uid . ":" . $monid . ":" . $het . ":" . $name;
            $binding_hash{$bindage} = 1;
          }
        }
      }

      # reset the parameters.
      $monid = "";
      $het = "";
      $name = "";
      $uniprots = "";
    }
    else {
      # get the parameter value.
      my $pval = trim ($line);

      # see if we have a parm name.
      if (defined ($parmName)) {
        # see if we have a value.
        unless ($pval eq "n/a") {
          # see if we are in the proper parm name.
          if ($parmName eq "BindingDB monomerid") {
            # set the value.
            $monid = $pval;
          }
          elsif ($parmName eq "HET ID") {
            # set the value.
            $het = $pval;
          }
          elsif ($parmName eq "BindingDB Display Name") {
            # set the value.
            $name = $pval;
          }
          elsif (index ($parmName, "UniProtKB Accession Number") == 0) {
            # append to the value.
            $uniprots = $uniprots . " " . $pval;
          }
        }
      }
    }
  }

  # close the binding database.
  close (BINDING);
}

# binding_dump_hash: dumps the hash into the database.
sub binding_dump_hash {
  # loop through the bindage hash.
  foreach my $bindage (keys (%binding_hash)) {

    # get the fields.
    my @fields = split (/:/, $bindage);
    my $uid = $fields[0];
    my $monid = $fields[1];
    my $het = $fields[2];
    my $name = $fields[3];

    # add this linkage to the database.
    sql_insert ("binding",
                sql_validate ($uid),
                sql_validate ($monid),
                sql_validate ($het),
                sql_validate ($name));
  }
}

# cause its a module.
return 1;
