
# borg-assmiliator/Core.pm: Core assimilation functions perl module
# Copyright (C) 2011 Bradley Worley
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License,
# or (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the
# 
#     Free Software Foundation, Inc.
#     59 Temple Place, Suite 330
#     Boston, MA 02111-1307 USA

# make life difficult.
use warnings;
use strict;

# load our perl modules.
use Cwd;
use DBI;
use Net::HTTP;
use XML::LibXML;
use HTML::Parser;
use File::Basename;

# the variable to hold whether our dataset is pdac or not.
my $pdacIndicator;

# print_greeting: prints the borg greeting.
sub print_greeting {
  # just some prints.
  print "\n";
  print " [=================================]\n";
  print " |         WE ARE THE BORG         |\n";
  print " |  your data will be assimilated  |\n";
  print " [=================================]\n\n";
}

# trim: strips whitespace from beginning and end of a string.
sub trim {
  # get the argument.
  my $string = shift (@_);

  # trim whitespace.
  $string =~ s/^\s+//;
  $string =~ s/\s+$//;

  # return the trimmed string.
  return $string;
}

# rtrim: strips trailing whitespace from end of a string.
sub rtrim {
  # get the argument.
  my $string = shift (@_);

  # trim it.
  $string =~ s/\s+$//;

  # return the trimmed string.
  return $string;
}

# ltrim: strips leading whitespace from the beginning of a string.
sub ltrim {
  # get the argument.
  my $string = shift (@_);

  # trim it.
  $string =~ s/^\s+//;

  # return the trimmed string.
  return $string;
}

# ensure_directory: makes sure the directory named exists.
sub ensure_directory {
  # get the argument.
  my $dir = shift (@_);

  # make sure the directory exists.
  unless (-d $dir) {
    # make the directory.
    print "creating directory " . $dir . "\n";
    mkdir ($dir);
  }
}

# ensure_directories: makes sure the directories exist.
sub ensure_directories {
  print "checking for directory structure for data import.\n";
  while (my $dir = shift (@_)) {
    ensure_directory ($dir);
  }
}

# http_get: simply downloads the requested resource to the requested file.
sub http_get {
  # get the arguments.
  my $host = shift (@_);
  my $resource = shift (@_);
  my $dir = shift (@_);

  # get the full filename.
  my $file = $dir . basename ($resource);

  # exit if the file exists.
  return $file if (-e $file);

  # output info.
  print " GET " . $host . " " . $resource . ": "; $|++;

  # send a request for the file.
  my $sock = Net::HTTP->new (Host => $host) or die $@;
  $sock->write_request (GET => $resource,
                        "User-Agent" => "Mozilla/5.0");
  my ($code, $mess, %heads) = $sock->read_response_headers ();

  # output a status message.
  print $code . " " . $mess . "\n"; $|++;

  # has this been moved?
  if ($code == 301) {
    # determine the new resource.
    if (defined ($heads{"location"})) {
      $resource = $heads{"location"};
    }
    elsif (defined ($heads{"Location"})) {
      $resource = $heads{"Location"};
    }
    elsif (defined ($heads{"LOCATION"})) {
      $resource = $heads{"LOCATION"};
    }
    else {
      die ("Cannot determine permanently moved filename");
    }

    # download the file.
    return http_get ($host, $resource, $dir);
  }

  # open the file for writing.
  open (GETFILE, ">", $file)
    or die ("could not open " . $file . " for writing");

  # loop through the downloaded content.
  while (1) {
    # read into the buffer.
    my $buffer;
    my $nbytes = $sock->read_entity_body ($buffer, 1024);
    die ("read failed: $!") unless defined ($nbytes);
    last unless $nbytes;

    # write to the file.
    print GETFILE $buffer;
  }

  # close the output file.
  close (GETFILE);

  # return ok.
  return $file;
}

# cause its a module.
return 1;
