
# borg-assmiliator/DIP.pm: DIP assimilation perl module
# Copyright (C) 2011 Bradley Worley
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License,
# or (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the
# 
#     Free Software Foundation, Inc.
#     59 Temple Place, Suite 330
#     Boston, MA 02111-1307 USA

# make life difficult.
use warnings;
use strict;

# setup the dip file locations.
my $dip_file = "dip.txt";

# declare the dip hash.
my %dip_link_hash;
my %dip_orig_hash;

# dip_build_hash: builds the hash for dumping into the database.
sub dip_build_hash {
  # get the working dir.
  my $cwd = getcwd ();

  # output a quick info message.
  print "parsing pre-downloaded dip interaction database.\n";

  # open the dip interaction database.
  open (DIPS, "<", $cwd . "/" . $dip_file);

  # loop through the file.
  while (my $line = <DIPS>) {
    # try to parse out the interaction.
    if ($line =~ /^(DIP-[^\t]*)\t(DIP-[^\t]*)\t.*$/) {
      # get the parsed interactor strings.
      my $intA = trim ($1);
      my $intB = trim ($2);

      # split the interactor strings.
      my @idsA = split (/\|/, $intA);
      my @idsB = split (/\|/, $intB);

      # declare uniprots.
      my $uidA;
      my $uidB;

      # loop through the first split string.
      foreach my $ident (@idsA) {
        # check if its a uniprot.
        if (index ($ident, "uniprotkb:") == 0) {
          my @idents = split (/:/, $ident);
          $uidA = $idents[1];
        }
      }

      # loop through the second split string.
      foreach my $ident (@idsB) {
        # check if its a uniprot.
        if (index ($ident, "uniprotkb:") == 0) {
          my @idents = split (/:/, $ident);
          $uidB = $idents[1];
        }
      }

      # did we find two uniprots?
      if (defined ($uidA) && defined ($uidB)) {
        # make sure they exist. does this speed things up?
        next unless uniprot_is_valid ($uidA);
        next unless uniprot_is_valid ($uidB);

        # make sure both are reviewed.
        uniprot_yank_from_db ($uidA);
        next unless uniprot_is_reviewed ($uidA);
        uniprot_yank_from_db ($uidB);
        next unless uniprot_is_reviewed ($uidB);

        # store the linkage in the hash.
        my $linkage = $uidA . ":" . $uidB;
        $dip_link_hash{$linkage} = 1;

        # determine whether the two uniprots are originals.
        unless (defined ($dip_orig_hash{$linkage})) {
          $dip_orig_hash{$linkage} = 1
            if (uniprot_is_original ($uidA)
            and uniprot_is_original ($uidB));
        }
      }
    }
  }

  # close the dip database.
  close (DIPS);
}

# dip_dump_hash: dumps the hash into the database.
sub dip_dump_hash {
  # loop through the linkage hash.
  foreach my $linkage (keys (%dip_link_hash)) {

    # get the actors.
    my @actors = split (/:/, $linkage);

    # generate the originality string.
    my $original = "false";
    $original = "true" if defined ($dip_orig_hash{$linkage});

    # add this linkage to the database.
    sql_insert ("dip", $actors[0], $actors[1], $original);
  }
}

# cause its a module.
return 1;
