
# borg-assmiliator/PDB.pm: PDB assimilation perl module
# Copyright (C) 2011 Bradley Worley
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License,
# or (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the
# 
#     Free Software Foundation, Inc.
#     59 Temple Place, Suite 330
#     Boston, MA 02111-1307 USA

# make life difficult.
use warnings;
use strict;

# setup the database location.
my $pdb_file;
my $pdb_http_host = "www.pdb.org";
my $pdb_http_resource = "/pdb/rest/ligandInfo?structureId=";
my $pdb_http_file = "ligandInfo?structureId=";
my $pdb_db_dir = "pdb";

# the hash of ligand exclusions.
my %pdb_ligand_exclusions;

# pdb_download_ligands: downloads a pdb ligands file.
sub pdb_download_ligands {
  # get the arguments.
  my $cwd = getcwd ();
  my $pdbID = shift (@_);

  # does the pdb db dir exist?
  mkdir ($cwd . "/" . $pdb_db_dir)
    unless (-e $cwd . "/" . $pdb_db_dir);

  # check if we have the file.
  $pdb_file = $cwd . "/" . $pdb_db_dir . "/" . $pdbID . "-ligands";
  unless (-e $pdb_file) {
    # download the file.
    http_get ($pdb_http_host, $pdb_http_resource . $pdbID,
              $cwd . "/");

    # move the file into the correct location.
    rename ($cwd . "/" . $pdb_http_file . $pdbID, $pdb_file);
  }
}

# pdb_build_exclusion_hash: builds the hash of excluded ligands.
sub pdb_build_exclusion_hash {
  # get the working dir.
  my $cwd = getcwd ();
  my @exclusions;

  # build the list of excluded pdb ligands.
  print "checking for list of excluded pdb ligand identifiers.\n";
  die ("where is my exclude.txt?!?!") unless (-e $cwd . "/exclude.txt");

  # output a helpful message.
  print "building list of excluded pdb ligand identifiers... ";

  # open the file.
  open (EXCLUDE, "<", $cwd . "/exclude.txt");

  # loop through the file.
  while (my $excluded = <EXCLUDE>) { push (@exclusions, trim ($excluded)); }

  # output the number of found exclusions.
  print @exclusions . " found.\n";

  # build the hash.
  my %pdb_ligand_exclusions = map { $_ => 1 } @exclusions;

  # close the file.
  close (EXCLUDE);
}

# pdb_build_tables: builds the mysql table of ligands.
sub pdb_build_tables {
  # get the variables.
  my $cwd = getcwd ();

  # build the list of pdb xml files.
  opendir (LIGANDS, $cwd . "/" . $pdb_db_dir)
    or die ("failed to open " . $cwd . "/" . $pdb_db_dir);
  my @pdbFiles = readdir (LIGANDS);
  closedir (LIGANDS);

  # output a quick info message.
  print "ok... parsing " . @pdbFiles . " downloaded pdb ligand info files.\n";

  # loop through the pdb ligand xml files.
  foreach my $pdbFile (@pdbFiles) {
    # dont process the special files.
    next if ($pdbFile eq "." or $pdbFile eq "..");

    # dont process non-ligand files.
    next unless ($pdbFile =~ /ligands/);

    # get the pdb id string.
    $pdb_file = $cwd . "/" . $pdb_db_dir . "/" . $pdbFile;
    my $pdbID = $pdbFile;
    $pdbID =~ s/\.txt//g;

    # ligands for each pdb entry.
    my @ligIDs;

    # load the xml parser and grab its ligands.
    my $xmlParser = XML::LibXML->new ();
    my $xmlTree = $xmlParser->parse_file ($pdb_file);
    my $xmlRoot = $xmlTree->documentElement ();
    my @xmlLigands = $xmlRoot->getElementsByTagName ("ligand");

    # look through the ligand elements.
    foreach my $xmlLigand (@xmlLigands) {
      # try to get teh chemical id attribute. ;)
      my $ligID = $xmlLigand->getAttribute ("chemicalID");

      # if we got a ligand id, write it to our ids.
      push (@ligIDs, $ligID) unless ($pdb_ligand_exclusions{trim ($ligID)});
    }

    # add the ligands to the database.
    $pdbID =~ s/^(....)-lig.*/$1/g;
    foreach my $ligID (@ligIDs)
    { sql_insert ("pdbLigands",
                  sql_validate ($pdbID),
                  sql_validate ($ligID)); }
  }
}

# cause its a module.
return 1;
