
# borg-assmiliator/Pfam.pm: Pfam assimilation perl module
# Copyright (C) 2011 Bradley Worley
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License,
# or (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the
# 
#     Free Software Foundation, Inc.
#     59 Temple Place, Suite 330
#     Boston, MA 02111-1307 USA

# make life difficult.
use warnings;
use strict;

# load our perl modules.
use Cwd;
use Net::FTP;
use IO::Uncompress::Gunzip qw(gunzip $GunzipError);

# setup the pfam ftp location.
my $pfam_ftp_host = "ftp.sanger.ac.uk";
my $pfam_ftp_dir = "pub/databases/Pfam/current_release/database_files";
my $pfam_ftp_file = "pfamA.txt.gz";
my $pfam_db_file = "pfam.txt";

# the global hash table.
my %pfam_descs;

# pfam_get_db: downloads the database, if necessary.
sub pfam_get_db {
  # get the arguments.
  my $cwd = getcwd ();

  # declare the list.
  my %pfamDescs;

  # do we already have the pfam file?
  print "checking for local copy of pfam descriptions.\n";
  return 1 if (-e $cwd . "/" . $pfam_db_file);

  # don't redownload this large file.
  print "downloading a local copy of pfam database.\n";
  unless (-e $cwd . "/" . $pfam_ftp_file) {
    # setup the ftp connection.
    my $ftp = Net::FTP->new ($pfam_ftp_host, Debug => 0)
      or die ("failed to connect to " . $pfam_ftp_host);

    # login.
    $ftp->login ("anonymous", "anonymous")
      or die ("failed to login to " . $pfam_ftp_host);

    # change to the proper directory.
    $ftp->cwd ($pfam_ftp_dir)
      or die ("failed to cwd into " . $pfam_ftp_dir);

    # get the uniprot file.
    $ftp->binary ();
    $ftp->get ($pfam_ftp_file, $cwd . "/" . $pfam_ftp_file)
      or die ("failed to get " . $pfam_ftp_file);
  }

  # unpack the gzip file.
  print "unpacking local copy of pfam database.\n";
  gunzip $cwd . "/" . $pfam_ftp_file => $pfam_ftp_file . ".u"
    or die ("failed to unpack " . $pfam_ftp_file);

  # open the unzipped file.
  open (PFAMS, "<", $cwd . "/" . $pfam_ftp_file . ".u");

  # process the unzipped file.
  while (my $line = <PFAMS>) {
    # see if we can pull data from the line.
    if ($line =~ /^[0-9]*\t*([^\t]*)\t*[^\t]*\t*([^\t]*)\t*.*$/) {
      # get the values.
      my $pfamFam = trim ($1);
      my $pfamDesc = trim ($2);

      # add the pair to the hash table.
      $pfamDescs{$pfamFam} = $pfamDesc;
    }
  }

  # close the unzipped file.
  close (PFAMS);

  # open the description file.
  open (PFAMDESC, ">", $cwd . "/" . $pfam_db_file);

  # loop through the hash table.
  foreach my $pfamFam (keys (%pfamDescs)) {
    # output the pairing.
    print PFAMDESC $pfamFam . "," . $pfamDescs{$pfamFam} . "\n";
  }

  # close the description file.
  close (PFAMDESC);

  # remove the gzip file.
  unlink ($cwd . "/" . $pfam_ftp_file);
  unlink ($cwd . "/" . $pfam_ftp_file . ".u");
}

# pfam_read_db: reads the pfam table into a hash table.
sub pfam_read_db {
  # get the arguments.
  my $cwd = getcwd ();

  # do we already have the pfam file?
  print "reading in local copy of pfam descriptions.\n";
  return 0 unless (-e $cwd . "/" . $pfam_db_file);

  # open the description file.
  open (PFAMDESC, "<", $cwd . "/" . $pfam_db_file);

  # read the description file.
  while (my $line = <PFAMDESC>) {
    # see if we can pull data from the line.
    if ($line =~ /^([^,]*),(.*)$/) {
      # get the values.
      my $pfamFam = trim ($1);
      my $pfamDesc = trim ($2);

      # add the pair to the hash table.
      $pfam_descs{$pfamFam} = $pfamDesc;
    }
  }

  # close the description file.
  close (PFAMDESC);
}

# pfam_get_desc: get the description text of a Pfam family.
sub pfam_get_desc {
  # get the argument.
  my $fam = shift (@_);

  # just return out of the hash table.
  return $pfam_descs{$fam};
}

# cause its a module.
return 1;
