
# borg-assmiliator/SQL.pm: main SQL functions perl module
# Copyright (C) 2011 Bradley Worley
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License,
# or (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the
# 
#     Free Software Foundation, Inc.
#     59 Temple Place, Suite 330
#     Boston, MA 02111-1307 USA

# the mysql database connection details.
my $sqlDatabase = "borg_np";
my $sqlHostname = "bionmr-16";
my $sqlUsername = "bworley";
my $sqlPassword = "mysqlpass";

# the mysql data source name.
my $sqlDatasource = "DBI:mysql:" . $sqlDatabase . ":" . $sqlHostname;
my $sqlHandle;
my $sqlQuery;

# sql_connect: connects to the database.
sub sql_connect {
  # connect to the database.
  $sqlHandle = DBI->connect ($sqlDatasource,
                             $sqlUsername,
                             $sqlPassword,
                             { RaiseError => 1,
                               PrintError => 0,
                               AutoCommit => 1 })
    or die ("could not connect to mysql database");

  # ensure that we automagically reconnect to the server.
  $sqlHandle->{mysql_auto_reconnect} = 1;
}

# sql_disconnect: disconnects from the database.
sub sql_disconnect {
  # disconnect from the database.
  $sqlHandle->disconnect ()
    or die ("could not disconnect from mysql database");
}

# sql_check_table: checks to make sure a table exists.
sub sql_check_table {
  # get the argument.
  my $table = shift (@_);

  # evaluate the description of the table.
  eval { $sqlHandle->do ("describe " . $table) };
  die ("sql_check_table failed to find table '" . $table . "' " . $@) if ($@);
}

# sql_check_tables: checks a listing of tables, all at once.
sub sql_check_tables {
  while (my $table = shift (@_)) {
    sql_check_table ($table);
  }
}

# sql_is_table_nonempty: checks what you think it checks.
sub sql_is_table_nonempty {
  my $table = shift (@_);
  $sqlQuery = $sqlHandle->prepare ("select * from " . $table);
  $sqlQuery->execute ();
  my $rows = $sqlQuery->rows ();
  $sqlQuery->finish ();
  return ($rows > 0);
}

# sql_num_query_results: returns the number of query results.
sub sql_num_query_results {
  my $q = shift (@_);
  $sqlQuery = $sqlHandle->prepare ($q);
  $sqlQuery->execute ();
  my $rows = $sqlQuery->rows ();
  $sqlQuery->finish ();
  return $rows;
}

# sql_validate: validates a sql value.
sub sql_validate {
  my $value = shift (@_);
  $value = "NULL" unless defined ($value);
  $value = "NULL" if ($value eq "");
  $value =~ s/\'//g;
  return $value;
}

# sql_insert: inserts a row of data, kind of clumsily.
sub sql_insert {
  # get the arguments.
  my $table = shift (@_);
  my $valueString;
  my @values;

  # get the values from the arguments.
  while (my $value = shift (@_)) {
    push (@values, $value);
  }

  # build the command.
  my $prefix = "insert into " . $table . " values(";
  for (my $i = 0; $i < @values; $i++) {
    if ($values[$i] eq "NULL") {
      if ($i == @values - 1) { $valueString .= $values[$i]; }
      else { $valueString .= $values[$i] . ", "; }
    }
    else {
      if ($i == @values - 1) { $valueString .= "'" . $values[$i] . "'"; }
      else { $valueString .= "'" . $values[$i] . "', "; }
    }
  }
  my $suffix = ")";

  # run the command.
  $sqlHandle->do ($prefix . $valueString . $suffix)
    or die ("failed to insert row into table " . $table);
}

# cause its a module.
return 1;
