
# borg-assmiliator/Sources.pm: data sources assimilation perl module
# Copyright (C) 2011 Bradley Worley
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License,
# or (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the
# 
#     Free Software Foundation, Inc.
#     59 Temple Place, Suite 330
#     Boston, MA 02111-1307 USA

# make life difficult.
use warnings;
use strict;

# sources_build_table: builds the sources table.
sub sources_build_table {
  # get the args.
  my $cwd = getcwd ();

  # connect to the mysql database.
  sql_connect ();

  # output.
  print "checking for a complete sources table.\n";

  # check the sources table.
  return 1 if (sql_is_table_nonempty ("sources"));
  return 0 unless (-e $cwd . "/sources.txt");

  # output.
  print "ok... building the sources table.\n";

  # open the sources file.
  open (SOURCES, "<", $cwd . "/sources.txt");

  # loop through the sources file.
  while (my $line = <SOURCES>) {
    # get the fields.
    my @fields = split (/,/, $line);

    # skip the first line.
    next if ($fields[0] eq "UniProt ID");

    # save the fields into variables.
    my $uid = sql_validate (uc (trim ($fields[0])));
    my $sid = sql_validate (trim ($fields[1]));
    my $type = sql_validate (lc (trim ($fields[2])));
    my $reg = sql_validate (lc (trim ($fields[3])));
    my $ref = sql_validate (trim ($fields[4]));

    # check the regulation field.
    if ($reg eq "n/a") { $reg = "NULL"; }

    # save to the sources table.
    sql_insert ("sources", $uid, $sid, $type, $reg, $ref);
  }

  # disconnect from the mysql database.
  sql_disconnect ();

  # close the sources file.
  close (SOURCES);
}

# cause its a module.
return 1;
