
# borg-assmiliator/eggNOG.pm: eggNOG assimilation perl module
# Copyright (C) 2011 Bradley Worley
# 
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License,
# or (at your option) any later version.
# 
# This program is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the
# 
#     Free Software Foundation, Inc.
#     59 Temple Place, Suite 330
#     Boston, MA 02111-1307 USA

# make life difficult.
use warnings;
use strict;

# load our perl modules.
use Cwd;
use Net::FTP;
use IO::Uncompress::Gunzip qw(gunzip $GunzipError);

# setup the eggnog http location.
my $eggnog_http_host = "eggnog.embl.de";
my $eggnog_http_dir = "/download";
my $eggnog_http_file = "orthgroups.mapping.txt.gz";
my $eggnog_db_file = "eggnog.txt";

# the global hash table.
my %eggnog_descs;

# eggnog_get_db: downloads the database, if necessary.
sub eggnog_get_db {
  # get the arguments.
  my $cwd = getcwd ();

  # declare the list.
  my %nogDescs;

  # do we already have the uniprot file?
  print "checking for local copy of eggnog descriptions.\n";
  return 1 if (-e $cwd . "/" . $eggnog_db_file);

  # don't redownload this large file.
  print "downloading a local copy of eggnog database.\n";
  unless (-e $cwd . "/" . $eggnog_http_file) {
    # download the file.
    http_get ($eggnog_http_host, $eggnog_http_dir . "/" . $eggnog_http_file,
              $cwd . "/") or die ("failed to download eggnog database");
  }

  # unpack the gzip file.
  print "unpacking local copy of eggnog database.\n";
  gunzip $cwd . "/" . $eggnog_http_file => $eggnog_http_file . ".u"
    or die ("failed to unpack " . $eggnog_http_file);

  # open the unzipped file.
  open (EGGNOGS, "<", $cwd . "/" . $eggnog_http_file . ".u");

  # process the unzipped file.
  while (my $line = <EGGNOGS>) {
    # see if we can pull data from the line.
    if ($line =~ /^[^\t]*\t[^\t]*\t[^\t]*\t([^\t]*)\t([^\t]*)$/) {
      # get the values.
      my $nogGroup = trim ($1);
      my $nogDesc = trim ($2);

      # add the pair to the hash table.
      $nogDescs{$nogGroup} = $nogDesc;
    }
  }

  # close the unzipped file.
  close (EGGNOGS);

  # open the description file.
  open (EGGDESC, ">", $cwd . "/" . $eggnog_db_file);

  # loop through the hash table.
  foreach my $nogGroup (keys (%nogDescs)) {
    # output the pairing.
    print EGGDESC $nogGroup . "," . $nogDescs{$nogGroup} . "\n";
  }

  # close the description file.
  close (EGGDESC);

  # remove the gzip file.
  unlink ($cwd . "/" . $eggnog_http_file);
  unlink ($cwd . "/" . $eggnog_http_file . ".u");
}

# eggnog_read_db: reads the eggnog table into a hash table.
sub eggnog_read_db {
  # get the arguments.
  my $cwd = getcwd ();

  # print a nice message.
  print "reading in local copy of eggnog database.\n";
  return 0 unless (-e $cwd . "/" . $eggnog_db_file);

  # open the description file.
  open (EGGDESC, "<", $cwd . "/" . $eggnog_db_file);

  # read the description file.
  while (my $line = <EGGDESC>) {
    # see if we can pull data from the line.
    if ($line =~ /^([^,]*),(.*)$/) {
      # get the values.
      my $nogGroup = trim ($1);
      my $nogDesc = trim ($2);

      # add the pair to the hash table.
      $eggnog_descs{$nogGroup} = $nogDesc;
    }
  }

  # close the description file.
  close (EGGDESC);
  return 1;
}

# eggnog_get_desc: get the description text of an eggNOG group.
sub eggnog_get_desc {
  # get the argument.
  my $nog = shift (@_);

  # just return out of the hash table.
  return $eggnog_descs{$nog};
}

# cause its a module.
return 1;
